@@ -40,6 +40,8 @@
 #ifndef PHONON_NO_DBUS
 #include <QtDBus/QtDBus>
 #endif
+#include <QtGui/QApplication>
+#include <QtGui/QMessageBox>
 
 QT_BEGIN_NAMESPACE
 
@@ -66,6 +68,9 @@ class FactoryPrivate : public Phonon::Factory::Sender
         QList<QObject *> objects;
         QList<MediaNodePrivate *> mediaNodePrivateList;
 
+        /// Whether the 'backend changed, please restart' message was shown.
+        bool m_backendChangedNotified;
+
     private Q_SLOTS:
         /**
          * This is called via DBUS when the user changes the Phonon Backend.
@@ -114,9 +119,17 @@ bool FactoryPrivate::createBackend()
 {
 #ifndef QT_NO_LIBRARY
     Q_ASSERT(m_backendObject == 0);
+
+    // If the user defines a backend with PHONON_BACKEND this overrides the
+    // platform plugin (because we cannot influence its lookup priority) and
+    // consequently will try to find/load the defined backend manually.
+    const QByteArray backendEnv = qgetenv("PHONON_BACKEND");
+
 #ifndef QT_NO_PHONON_PLATFORMPLUGIN
     PlatformPlugin *f = globalFactory->platformPlugin();
-    if (f) {
+    if (f && backendEnv.isEmpty()) {
+        // TODO: it would be very groovy if we could add a param, so that the
+        // platform could also try to load the defined backend as preferred choice.
         m_backendObject = f->createBackend();
     }
 #endif //QT_NO_PHONON_PLATFORMPLUGIN
@@ -150,10 +163,6 @@ bool FactoryPrivate::createBackend()
             }
 #endif
 
-#ifdef __GNUC__
-#warning TODO - phonon should really parse cmdline args
-#endif // __GNUC__
-            const QByteArray backendEnv = qgetenv("PHONON_BACKEND");
             if (!backendEnv.isEmpty()) {
                 pDebug() << "trying to load:" << backendEnv << "as first choice";
                 const int backendIndex = plugins.indexOf(QRegExp(backendEnv + ".*"));
@@ -209,10 +218,11 @@ bool FactoryPrivate::createBackend()
 FactoryPrivate::FactoryPrivate()
     :
 #ifndef QT_NO_PHONON_PLATFORMPLUGIN
-    m_platformPlugin(0),
-    m_noPlatformPlugin(false),
+      m_platformPlugin(0),
+      m_noPlatformPlugin(false),
 #endif //QT_NO_PHONON_PLATFORMPLUGIN
-    m_backendObject(0)
+      m_backendObject(0),
+      m_backendChangedNotified(false)
 {
     // Add the post routine to make sure that all other global statics (especially the ones from Qt)
     // are still available. If the FactoryPrivate dtor is called too late many bad things can happen
@@ -297,30 +307,16 @@ void Factory::deregisterFrontendObject(MediaNodePrivate *bp)
 #ifndef PHONON_NO_DBUS
 void FactoryPrivate::phononBackendChanged()
 {
-    if (m_backendObject) {
-        for (int i = 0; i < mediaNodePrivateList.count(); ++i) {
-            mediaNodePrivateList.at(i)->deleteBackendObject();
-        }
-        if (objects.size() > 0) {
-            pDebug() << "WARNING: we were asked to change the backend but the application did\n"
-                "not free all references to objects created by the factory. Therefore we can not\n"
-                "change the backend without crashing. Now we have to wait for a restart to make\n"
-                "backendswitching possible.";
-            // in case there were objects deleted give 'em a chance to recreate
-            // them now
-            for (int i = 0; i < mediaNodePrivateList.count(); ++i) {
-                mediaNodePrivateList.at(i)->createBackendObject();
-            }
-            return;
-        }
-        delete m_backendObject;
-        m_backendObject = 0;
-    }
-    createBackend();
-    for (int i = 0; i < mediaNodePrivateList.count(); ++i) {
-        mediaNodePrivateList.at(i)->createBackendObject();
-    }
+    if (m_backendChangedNotified)
+        return;
+
+    QMessageBox::information(qApp->activeWindow(),
+                             tr("Restart Application"),
+                             tr("You changed the backend of the Phonon multimedia system.\n\n"
+                                "To apply this change you will need to"
+                                " restart '%1'.").arg(qAppName()));
     emit backendChanged();
+    m_backendChangedNotified = true;
 }
 #endif //PHONON_NO_DBUS
 
